@@ -1,14 +1,14 @@
 ###############################################################################
 # Methods for Discrete Distributions
 ###############################################################################
-                          
+
 ## (c) Matthias Kohl: revised P.R. 030707
 
 DiscreteDistribution <- function(supp, prob, .withArith = FALSE,
      .withSim = FALSE, .lowerExact = TRUE, .logExact = FALSE,
-     .DistrCollapse = 
+     .DistrCollapse =
                   getdistrOption("DistrCollapse"),
-     .DistrCollapse.Unique.Warn = 
+     .DistrCollapse.Unique.Warn =
                   getdistrOption("DistrCollapse.Unique.Warn"),
      .DistrResolution = getdistrOption("DistrResolution"),
      Symmetry = NoSymmetry()){
@@ -30,7 +30,7 @@ DiscreteDistribution <- function(supp, prob, .withArith = FALSE,
         if(!all(prob >= 0))
             stop("'prob' contains values < 0")
     }
-    
+
     o <- order(supp)
     supp <- supp[o]
     prob <- prob[o]
@@ -40,11 +40,11 @@ DiscreteDistribution <- function(supp, prob, .withArith = FALSE,
        if (len>1 && min(diff(supp))< .DistrResolution){
            erg <- .DistrCollapse(supp, prob, .DistrResolution)
            if (len>length(erg$prob) && .DistrCollapse.Unique.Warn)
-               warning("collapsing to unique support values")         
+               warning("collapsing to unique support values")
            prob <- erg$prob
            supp <- erg$supp
        }
-    }else{    
+    }else{
        usupp <- unique(supp)
        if(length(usupp) < len){
           if(.DistrCollapse.Unique.Warn)
@@ -61,8 +61,14 @@ DiscreteDistribution <- function(supp, prob, .withArith = FALSE,
     }
     rm(len)
 
-    rfun <- function(n){
-        sample(x = supp, size = n, replace = TRUE, prob = prob)
+    if(length(supp) == 1){
+      rfun <- function(n){
+        rep(supp, n)
+      }
+    }else{
+      rfun <- function(n){
+          sample(x = supp, size = n, replace = TRUE, prob = prob)
+      }
     }
 
     dfun <- .makeDNew(supp, prob, Cont = FALSE)
@@ -84,7 +90,7 @@ setMethod("p.l", "DiscreteDistribution", function(object){
        if (.inArgs("lower.tail", p(object))){
            function(q, lower.tail = TRUE, log.p = FALSE){
                 px <- p(object)(q, lower.tail = lower.tail)
-                o.warn <- getOption("warn"); 
+                o.warn <- getOption("warn");
                 on.exit(options(warn=o.warn))
                 options(warn = -2)
                 dx <- d(object)(.setEqual(q, support(object)))
@@ -220,7 +226,7 @@ setMethod("q.r", "DiscreteDistribution", function(object){
 
 setMethod("+", c("DiscreteDistribution","DiscreteDistribution"),
 function(e1,e2){
-            
+
             if(length(support(e1))==1) return(e2+support(e1))
             if(length(support(e2))==1) return(e1+support(e2))
             e1.L <- as(e1, "LatticeDistribution")
@@ -232,7 +238,7 @@ function(e1,e2){
                    if (abs(abs(w1)-abs(w2))<getdistrOption("DistrResolution") ||
                        W[2] %% W[1] < getdistrOption("DistrResolution") )
                        return(e1.L + e2.L)
-                  } 
+                  }
             .convDiscrDiscr(e1,e2)})
 
 setMethod("+", c("Dirac","DiscreteDistribution"),
@@ -247,7 +253,7 @@ setMethod("*", c("DiscreteDistribution","numeric"),
                                  Distr@X0 <- e1
 
                               if(is(e1@Symmetry,"SphericalSymmetry"))
-                                 Distr@Symmetry <- 
+                                 Distr@Symmetry <-
                                    SphericalSymmetry(SymmCenter(e1@Symmetry)*e2)
 
                               Distr
@@ -258,7 +264,7 @@ setMethod("+", c("DiscreteDistribution","numeric"),
                                  Distr@X0 <- e1
 
                               if(is(e1@Symmetry,"SphericalSymmetry"))
-                                 Distr@Symmetry <- 
+                                 Distr@Symmetry <-
                                    SphericalSymmetry(SymmCenter(e1@Symmetry)+e2)
 
                               Distr
@@ -268,17 +274,17 @@ setMethod("*", c("AffLinDiscreteDistribution","numeric"),
            function(e1, e2) {
                 Distr <- .multm(e1,e2, "AffLinDiscreteDistribution")
                 if(is(e1@Symmetry,"SphericalSymmetry"))
-                      Distr@Symmetry <- 
+                      Distr@Symmetry <-
                         SphericalSymmetry(SymmCenter(e1@Symmetry)*e2)
-                Distr                
+                Distr
                 })
 setMethod("+", c("AffLinDiscreteDistribution","numeric"),
            function(e1, e2) {
                 Distr <- .plusm(e1,e2, "AffLinDiscreteDistribution")
                 if(is(e1@Symmetry,"SphericalSymmetry"))
-                      Distr@Symmetry <- 
+                      Distr@Symmetry <-
                         SphericalSymmetry(SymmCenter(e1@Symmetry)*e2)
-                Distr                
+                Distr
                 })
 
 ## Group Math for discrete distributions
@@ -298,47 +304,47 @@ setMethod("Math", "Dirac",
 
 ## exact: abs for discrete distributions
 setMethod("abs", "DiscreteDistribution",function(x){
-       
+
        rnew <- function(n, ...){}
        body(rnew) <- substitute({ abs(g(n, ...)) },
                                       list(g = x@r))
-       
+
        xx <- x
        supportnew <- support(x)
-       
+
        isSym0 <- FALSE
        if(is(Symmetry(x),"SphericalSymmetry"))
           if(.isEqual(SymmCenter(Symmetry(x)),0))
-             isSym0 <- TRUE  
-       
+             isSym0 <- TRUE
+
        if(isSym0){
           supportnew <- supportnew[supportnew>=0]
-       
+
           .lowerExact = .lowerExact(x)
 
-          dxlog <- if("log" %in% names(formals(d(x)))) 
+          dxlog <- if("log" %in% names(formals(d(x))))
                         quote({dx <- d(xx)(x, log = TRUE)})
                    else quote({dx <-  log(d(xx)(x))})
-          pxlog <- if("log.p" %in% names(formals(p(x))) && 
-                       "lower.tail" %in% names(formals(p(x)))) 
+          pxlog <- if("log.p" %in% names(formals(p(x))) &&
+                       "lower.tail" %in% names(formals(p(x))))
                         quote({p(x)(q, lower.tail = FALSE, log.p = TRUE)})
                    else
                         quote({log(1-p(x)(q))})
 
 
-          qxlog <- if("lower.tail" %in% names(formals(q(x)))) 
+          qxlog <- if("lower.tail" %in% names(formals(q(x))))
                           quote({qx <- if(lower.tail)
                                           q(x)((1+p1)/2)
                                        else
-                                          q(x)(p1/2,lower.tail=FALSE)}) 
+                                          q(x)(p1/2,lower.tail=FALSE)})
                       else
                           quote({qx <- q(x)(if(lower.tail) (1+p1)/2 else 1-p1/2)})
-          if("lower.tail" %in% names(formals(q(x)))&& 
-             "log.p" %in% names(formals(q(x))))           
+          if("lower.tail" %in% names(formals(q(x)))&&
+             "log.p" %in% names(formals(q(x))))
               qxlog <- quote({qx <- if(lower.tail) q(x)((1+p1)/2)
                                        else
                                           q(x)(if(log.p)p-log(2)
-                                               else p1/2,lower.tail=FALSE,log.p=log.p)}) 
+                                               else p1/2,lower.tail=FALSE,log.p=log.p)})
 
 
           dnew <- function(x, log = FALSE){}
@@ -349,7 +355,7 @@ setMethod("abs", "DiscreteDistribution",function(x){
                     dx[x<0] <- if(log) -Inf else 0
                     return(dx)
                     }, list(dxlog0 = dxlog))
-            
+
           pnew <- function(q, lower.tail = TRUE, log.p = FALSE){}
           body(pnew) <- substitute({
                     if (!lower.tail){
@@ -359,7 +365,7 @@ setMethod("abs", "DiscreteDistribution",function(x){
                         px <- pmax(2 * p(x)(q) - 1,0)
                         if(log.p) px <- log(px)
                     }
-                    return(px)            
+                    return(px)
             }, list(pxlog0 = pxlog))
 
           qnew <- function(p, lower.tail = TRUE, log.p = FALSE){}
@@ -380,13 +386,13 @@ setMethod("abs", "DiscreteDistribution",function(x){
             dnew <- function(x, log = FALSE){
                     o.warn <- getOption("warn"); options(warn = -1)
                     on.exit(options(warn=o.warn))
-                    dx <- (x>=0) * d(xx)(x) + (x>0) * d(xx)(-x) 
+                    dx <- (x>=0) * d(xx)(x) + (x>0) * d(xx)(-x)
                     options(warn = o.warn)
                     if (log) dx <- log(dx)
                     return(dx)
                  }
-            
-            pxlow <- if("lower.tail" %in% names(formals(p(x)))) 
+
+            pxlow <- if("lower.tail" %in% names(formals(p(x))))
                         substitute({p(x)(q, lower=FALSE)})
                    else
                         substitute({1-p(x)(q)})
@@ -394,21 +400,21 @@ setMethod("abs", "DiscreteDistribution",function(x){
             pnew <- function(q, lower.tail = TRUE, log.p = FALSE){}
             body(pnew) <- substitute({
                     px <- if (lower.tail)
-                            (q>=0) * (p(x)(q) - p.l(x)(-q))                    
+                            (q>=0) * (p(x)(q) - p.l(x)(-q))
                           else pxlow0 + p.l(x)(-q)
                     if (log.p) px <- log(px)
                     return(px)
             }, list(pxlow0=pxlow))
 
             prob <- dnew(supportnew)
-            
-            qnew <- .makeQNew(supportnew, cumsum(prob), 
-                            rev(cumsum(rev(prob))), notwithLLarg = x@.withSim, 
+
+            qnew <- .makeQNew(supportnew, cumsum(prob),
+                            rev(cumsum(rev(prob))), notwithLLarg = x@.withSim,
                             min(supportnew), max(supportnew), Cont = FALSE)
-            
+
          }
          object <- new("DiscreteDistribution", r = rnew, p = pnew,
-                        q = qnew, d = dnew, support = supportnew, 
+                        q = qnew, d = dnew, support = supportnew,
                         .withSim = x@.withSim, .withArith = TRUE,
                         .lowerExact = .lowerExact(x))
          object
@@ -420,7 +426,7 @@ setMethod("exp", "DiscreteDistribution",
 
 
 ### preliminary to export special functions
-if (getRversion()>='2.6.0'){ 
+if (getRversion()>='2.6.0'){
 
 setMethod("log", "DiscreteDistribution",
            function(x, base = exp(1)) {
@@ -428,31 +434,31 @@ setMethod("log", "DiscreteDistribution",
                  call = sys.call(sys.parent(1)))$x))
            ep <- getdistrOption("TruncQuantile")
            basl <- log(base)
-           if(p(x)(0)>ep) 
+           if(p(x)(0)>ep)
                 stop(gettextf("log(%s) is not well-defined with positive probability ", xs))
            else return(.logm.d(x)/basl)})
 
 setMethod("log", "Dirac",
-          function(x, base = exp(1)){ 
+          function(x, base = exp(1)){
                        xs <- as.character(deparse(match.call(
                              call = sys.call(sys.parent(1)))$x))
-                       loc <- location(x) 
+                       loc <- location(x)
                        ep <- getdistrOption("TruncQuantile")
                        basl <- log(base)
-                       if(loc < ep) 
-                          stop(gettextf("log(%s) is not well-defined with positive probability ", xs))                       
+                       if(loc < ep)
+                          stop(gettextf("log(%s) is not well-defined with positive probability ", xs))
                        Dirac(log(loc)/basl)})
 
 setMethod("log10", "DiscreteDistribution",
           function(x) log(x = x, base = 10))
 
 setMethod("sign", "DiscreteDistribution",
-          function(x){ 
+          function(x){
           d0 <- d(x)(0)
-          DiscreteDistribution(supp=c(-1,0,1), 
+          DiscreteDistribution(supp=c(-1,0,1),
               prob=c(p(x)(-getdistrOption("TruncQuantile")),
                      d0,
-                     p(x)(getdistrOption("TruncQuantile"), lower=FALSE)))                     
+                     p(x)(getdistrOption("TruncQuantile"), lower=FALSE)))
           })
 
 
@@ -462,9 +468,9 @@ setMethod("digamma", "DiscreteDistribution",
             if(px0>0) stop("argument of 'digamma' must be concentrated on positive values")
             rnew <-  function(n, ...){}
             body(rnew) <- substitute({ digamma(g(n, ...)) }, list(g = x@r))
-            
-            object <- DiscreteDistribution( 
-                     supp=digamma(support(x)), 
+
+            object <- DiscreteDistribution(
+                     supp=digamma(support(x)),
                      prob=prob(x), .withArith = TRUE)
             object
           })
@@ -489,23 +495,22 @@ setMethod("gamma", "DiscreteDistribution",
 setMethod("sqrt", "DiscreteDistribution",
             function(x) x^0.5)
 
-}          
-setMethod("prob", "DiscreteDistribution", 
+}
+setMethod("prob", "DiscreteDistribution",
 function(object) {sp <- object@support
                   pr <- object@d(sp)
                   names(pr) <- paste(sp)
                   return(pr)
                   })
 ## Replace Methods
-setReplaceMethod("prob", "DiscreteDistribution",  
-                  function(object, value){ 
-                  return(DiscreteDistribution(supp = object@support, 
+setReplaceMethod("prob", "DiscreteDistribution",
+                  function(object, value){
+                  return(DiscreteDistribution(supp = object@support,
                              prob = value,
                             .withArith = object@.withArith,
                             .withSim = object@.withSim,
-                            .lowerExact = .lowerExact(object), 
+                            .lowerExact = .lowerExact(object),
                             .logExact = .logExact(object)))}
                   )
 
 
-          
\ No newline at end of file
